/*******************************************************************************
regarding camera Prosilica ec750c
------ Features report ------
OP   - one push capable
RC   - readout capable
O/OC - on/off capable
AC   - auto capable
MC   - manual capable
ABS  - absolute capable
-----------------------------
Brightness:
	RC  O/OC  MC  (active is: MAN)  
	Feature: OFF  min: 0 max 255
	current value is: 0
Exposure:
	NOT AVAILABLE
Sharpness:
	NOT AVAILABLE
White Balance:
	RC  MC  (active is: MAN)  AC  (active is: MAN)  OP  (active is: MAN)  
	min: 0 max 255
	B/U value: 100 R/V value: 83
Hue:
	NOT AVAILABLE
Saturation:
	NOT AVAILABLE
Gamma:
	RC  O/OC  MC  (active is: MAN)  
	Feature: OFF  min: 0 max 1
	current value is: 0
Shutter:
	RC  MC  (active is: MAN)  AC  (active is: MAN)  OP  (active is: MAN)  ABS  
	min: 1 max 4095
	current value is: 750
	absolute settings:
	 value: 0.015000
	 min: 0.000020
	 max: 1.000000
Gain:
	RC  O/OC  MC  (active is: MAN)  
	Feature: OFF  min: 0 max 48
	current value is: 0
Iris:
	NOT AVAILABLE
Focus:
	NOT AVAILABLE
Temperature:
	NOT AVAILABLE
Trigger:
	RC  O/OC  
	Feature: OFF  
	AvailableTriggerModes: 384 
	AvailableTriggerSources: 576 580 
	Polarity Change Capable: True
	Current Polarity: POS
	current mode: 384

	current source: 576
Trigger Delay:
	RC  O/OC  MC  (active is: MAN)  ABS  
	Feature: ON  min: 0 max 4095
	current value is: 0
	absolute settings:
	 value: 0.000000
	 min: 0.000000
	 max: 1.000000
White Shading:
	NOT AVAILABLE
Frame Rate:
	RC  O/OC  MC  (active is: MAN)  ABS  
	Feature: OFF  min: 20 max 600
	current value is: 0
	absolute settings:
	 value: 0.000000
	 min: 2.000000
	 max: 64.000000
Zoom:
	NOT AVAILABLE
Pan:
	NOT AVAILABLE
Tilt:
	NOT AVAILABLE
Optical Filter:
	NOT AVAILABLE
Capture Size:
	NOT AVAILABLE
Capture Quality:
	NOT AVAILABLE
*******************************************************************************/

#include "fireCamLib.h" 

#define PRINT_CAM_FEATURES_TO_FILE 0
#define PRINT_DEBUG_INFO_TO_SCREEN 0


/*-----------------------------------------------------------------------
 *  Releases the cameras and exits
 *-----------------------------------------------------------------------*/
void cleanup_and_exit(dc1394camera_t *camera)
{
    dc1394_video_set_transmission(camera, DC1394_OFF);
    dc1394_capture_stop(camera);
    dc1394_camera_free(camera);
    /*exit(1);*/
}

/*-----------------------------------------------------------------------
 *  Initializes the camera and have the camera start sending us data
 *-----------------------------------------------------------------------*/
int fireCamInit(int frameRate)
{
    
    unsigned int width, height,i,j;
    dc1394video_frame_t* frame=NULL;
    dc1394camera_list_t* list;
    dc1394featureset_t features;
    dc1394error_t err;

    d = dc1394_new();
    err=dc1394_camera_enumerate (d, &list);
    DC1394_ERR_RTN(err,"Failed to enumerate cameras");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera enumeration success\n");

    if (list->num == 0) {
        dc1394_log_error("No cameras found");
        return 1;
    }

    camera = dc1394_camera_new (d, list->ids[0].guid);
    if (!camera) {
        dc1394_log_error("Failed to initialize camera with guid %llx", list->ids[0].guid);
        return 1;
    }
    dc1394_camera_free_list (list);

    printf("\nINFO --> Using camera with GUID %"PRIx64"\n", camera->guid);
    
    /* reset bus to clear previously allocated ISO bandwidth */
    dc1394_reset_bus(camera);

    /* print available features to file if requested*/
    if(PRINT_DEBUG_INFO_TO_SCREEN && !PRINT_CAM_FEATURES_TO_FILE) printf("DEBUG --> camera feature printing disabled\n");
    if(PRINT_CAM_FEATURES_TO_FILE)
    {
    	FILE* featurefile;
        err = dc1394_feature_get_all(camera, &features);
        DC1394_ERR_RTN(err,"Failed to obtain feature set");
        featurefile = fopen("features.txt", "w");
        err = dc1394_feature_print_all(&features,featurefile);
        DC1394_ERR_RTN(err,"Failed to print feature set");
        fclose(featurefile);
	if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera features printed to file\n");
    }
    
    
    /* ensure camera is powered on */
    err = dc1394_camera_set_power(camera, DC1394_ON);
    DC1394_ERR_RTN(err,"Failed to power on camera.\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera powered on\n");
    
    /*-----------------------------------------------------------------------
     *  Set Features
     *  Below are all the features supported by the camera Prosilica ec750c
     *-----------------------------------------------------------------------*/
    
    /* reset to factory defaults first */
    err = dc1394_camera_reset(camera);
    DC1394_ERR_RTN(err,"Failed to reset camera to factory defaults.\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera reset to factory defaults\n");
    
    /* Brightness */
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_BRIGHTNESS, DC1394_FEATURE_MODE_MANUAL);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_BRIGHTNESS\n");
    err = dc1394_feature_set_value(camera, DC1394_FEATURE_BRIGHTNESS, 0);
    DC1394_ERR_RTN(err,"Failed to set feature value: DC1394_FEATURE_BRIGHTNESS\n");
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_BRIGHTNESS, DC1394_OFF);
    DC1394_ERR_RTN(err,"Failed to set feature on/off: DC1394_FEATURE_BRIGHTNESS\n");
    
    /* White Balance */ 
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_WHITE_BALANCE, DC1394_FEATURE_MODE_AUTO);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_WHITE_BALANCE\n");

    /* Gamma */ 
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_GAMMA, DC1394_FEATURE_MODE_MANUAL);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_GAMMA\n");
    err = dc1394_feature_set_value(camera, DC1394_FEATURE_GAMMA, 0);
    DC1394_ERR_RTN(err,"Failed to set feature value: DC1394_FEATURE_GAMMA\n");
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_GAMMA, DC1394_OFF);
    DC1394_ERR_RTN(err,"Failed to set feature on/off: DC1394_FEATURE_GAMMA\n");
    
    /* Shutter */ 
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_SHUTTER, DC1394_FEATURE_MODE_AUTO);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_SHUTTER\n");
    
    /* Gain */ 
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_GAIN, DC1394_FEATURE_MODE_MANUAL);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_GAIN\n");
    err = dc1394_feature_set_value(camera, DC1394_FEATURE_GAIN, 0);
    DC1394_ERR_RTN(err,"Failed to set feature value: DC1394_FEATURE_GAIN\n");
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_GAIN, DC1394_OFF);
    DC1394_ERR_RTN(err,"Failed to set feature on/off: DC1394_FEATURE_GAIN\n");
    
    /* Trigger */ 
    err = dc1394_external_trigger_set_mode(camera, DC1394_TRIGGER_MODE_0);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_TRIGGER\n");
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_TRIGGER, DC1394_OFF);
    DC1394_ERR_RTN(err,"Failed to set feature on/off: DC1394_FEATURE_TRIGGER\n");
    
    /* Trigger Delay */ 
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_TRIGGER_DELAY, DC1394_FEATURE_MODE_MANUAL);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_TRIGGER_DELAY\n");
    err = dc1394_feature_set_value(camera, DC1394_FEATURE_TRIGGER_DELAY, 0);
    DC1394_ERR_RTN(err,"Failed to set feature value: DC1394_FEATURE_TRIGGER_DELAY\n");
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_TRIGGER_DELAY, DC1394_OFF);
    DC1394_ERR_RTN(err,"Failed to set feature on/off: DC1394_FEATURE_TRIGGER_DELAY\n");
    
    /* Frame Rate */ 
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_FRAME_RATE, DC1394_FEATURE_MODE_MANUAL);
    DC1394_ERR_RTN(err,"Failed to set feature mode: DC1394_FEATURE_FRAME_RATE\n");
    err = dc1394_feature_set_value(camera, DC1394_FEATURE_FRAME_RATE, frameRate);
    DC1394_ERR_RTN(err,"Failed to set feature value: DC1394_FEATURE_FRAME_RATE\n");
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_FRAME_RATE, DC1394_ON);
    DC1394_ERR_RTN(err,"Failed to set feature on/off: DC1394_FEATURE_FRAME_RATE\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera framerate set to %d\n",frameRate);
    

    /*-----------------------------------------------------------------------
     *  Select Capture Mode
     *
     *  Modes supported by camera Prosilica ec750c:
     *  DC1394_VIDEO_MODE_FORMAT7_0
     *  DC1394_VIDEO_MODE_640x480_MONO8
     *  DC1394_VIDEO_MODE_640x480_MONO16
     *-----------------------------------------------------------------------*/
    
    uint32_t selected_mode = DC1394_VIDEO_MODE_FORMAT7_0;
    
    err=dc1394_video_set_mode(camera, selected_mode);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not set video mode\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera capture mode is set\n");


    /*-----------------------------------------------------------------------
     *  Set iso speed 
     *  Select maximum speed possible by firewire a specs
     *-----------------------------------------------------------------------*/
    err=dc1394_video_set_iso_speed(camera, DC1394_ISO_SPEED_400);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not set iso speed");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> camera ISO speed is set\n");

    
    /*-----------------------------------------------------------------------
     *  setup regoin of interest, packet size / framerate and color coding
     *
     *  Color codes supported by camera Prosilica ec750c:
     *  DC1394_COLOR_CODING_YUV411
     *  DC1394_COLOR_CODING_YUV422
     *  DC1394_COLOR_CODING_RGB8
     *  DC1394_COLOR_CODING_MONO8
     *  DC1394_COLOR_CODING_MONO16
     *  DC1394_COLOR_CODING_RAW8
     *  DC1394_COLOR_CODING_RAW16
     *  
     *-----------------------------------------------------------------------*/
    


    if(selected_mode == DC1394_VIDEO_MODE_FORMAT7_0)
    {
        int32_t num_packets;
        int32_t packet_size, px, py, width, height;
        dc1394color_coding_t color_mode;
        
        packet_size = DC1394_USE_RECOMMENDED; //4096
        
        err = dc1394_format7_set_roi(camera, DC1394_VIDEO_MODE_FORMAT7_0, DC1394_COLOR_CODING_RAW8, packet_size, 0, 0, FRAME_WIDTH, FRAME_HEIGHT);
        if(err != DC1394_SUCCESS) printf("\nERROR --> Could not set ROI, packet size, color coding\n");
        else printf("\nINFO --> ROI, packet size, color coding set\n");
        
        err = dc1394_format7_get_roi(camera, DC1394_VIDEO_MODE_FORMAT7_0, &color_mode, &packet_size, &px, &py, &width, &height);
        printf("INFO --> ROI: {offset_x offset_y width height} = {%d %d %d %d}\n",px, py, width, height);

    }
    else if(selected_mode == DC1394_VIDEO_MODE_640x480_MONO8 || selected_mode == DC1394_VIDEO_MODE_640x480_MONO16)
    {
        /* possible frame rates: DC1394_FRAMERATE_7_5, DC1394_FRAMERATE_15, DC1394_FRAMERATE_30, DC1394_FRAMERATE_60 */
        err=dc1394_video_set_framerate(camera, DC1394_FRAMERATE_60);
        DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"\nERROR --> Could not set framerate\n");
    }
    else
    {
        printf("\nERROR --> mode not supported by Prosilica ec750c\n");
        cleanup_and_exit(camera);
    }

    /*-----------------------------------------------------------------------
     *  setup capture
     *-----------------------------------------------------------------------*/
    
    err=dc1394_capture_setup(camera,3, DC1394_CAPTURE_FLAGS_DEFAULT);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"\nCould not setup camera-\nmake sure that the video mode and framerate are\nsupported by your camera\n");
    if(err == DC1394_SUCCESS) printf("\nINFO --> Capture setup success\n");
    else
    {
        dc1394_reset_bus(camera);
    }
    
    /*-----------------------------------------------------------------------
     *  have the camera start sending us data
     *-----------------------------------------------------------------------*/
    err=dc1394_video_set_transmission(camera, DC1394_ON);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"\nERROR --> Could not start camera iso transmission\n");
    if(err == DC1394_SUCCESS) printf("INFO --> Transmission setup success\n");
    return 0;
}


int grabOneFrame(uint8_t *image, uint64_t *timestamp)
{
    /*-----------------------------------------------------------------------
     *  capture one frame
     *-----------------------------------------------------------------------*/
    
    dc1394video_frame_t *frame=NULL;
    dc1394error_t err;
    int result = 0;
    int i;
    
    err=dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_WAIT, &frame);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"\nERROR --> Could not capture frame\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> frame capture success\n");
        result = frame->frames_behind;
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> number of frames behind: %d\n",frame->frames_behind);
    
    *timestamp = frame->timestamp;
    
    /* The Color Pattern layout of the camera Prosilica EC750C is BGGR, thus using DC1394_COLOR_FILTER_BGGR in Bayer decoding */
    err=dc1394_bayer_decoding_8bit(frame->image, image, FRAME_WIDTH, FRAME_HEIGHT, DC1394_COLOR_FILTER_BGGR, DC1394_BAYER_METHOD_NEAREST        )	;
//     err=dc1394_convert_to_RGB8(frame->image,image,FRAME_WIDTH, FRAME_HEIGHT,0,DC1394_COLOR_CODING_RAW8,8);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not convert frame to RGB\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> convert to RGB success\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN) printf("DEBUG --> frame size %d %d\n",frame->size[0],frame->size[1]);
    if(PRINT_DEBUG_INFO_TO_SCREEN) printf("DEBUG --> frame stride %d image bytes %d\n",frame->stride, frame->image_bytes);
    dc1394_capture_enqueue(camera, frame);
    
    return frame->frames_behind;
}
    

int grabLastFrame(uint8_t *image, uint64_t *timestamp)
{
    int result = 2;
    while(result == 2)
        result = grabOneFrame(image,timestamp);
    
    return result;
}
    
void flushBuffer()
{
    dc1394video_frame_t *frame=NULL;
    dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_POLL, &frame);
    while(frame != NULL)
        dc1394_capture_dequeue(camera, DC1394_CAPTURE_POLICY_POLL, &frame);
}

/*-----------------------------------------------------------------------
 *  stop data transmission and close camera
 *-----------------------------------------------------------------------*/
int fireCamTerm()
{
    dc1394error_t err;
    
    /*-----------------------------------------------------------------------
     *  stop data transmission
     *-----------------------------------------------------------------------*/
    err=dc1394_video_set_transmission(camera,DC1394_OFF);
    DC1394_ERR_CLN_RTN(err,cleanup_and_exit(camera),"Could not stop the camera?\n");
    if(PRINT_DEBUG_INFO_TO_SCREEN && err==DC1394_SUCCESS) printf("DEBUG --> Camera stopped\n");

    
    /*-----------------------------------------------------------------------
     *  close camera
     *-----------------------------------------------------------------------*/
    dc1394_capture_stop(camera);
    dc1394_camera_free(camera);
    dc1394_free (d);

    return 0;
}


/*-----------------------------------------------------------------------
 *  set the absolute shutter speed value
 *    return 1 on success, return 0 on failure
 *-----------------------------------------------------------------------*/
int setShutterSpeedAbs(float shutterSpeed, float stepTime)
{
    dc1394error_t err;
    dc1394feature_mode_t* mode;
    dc1394switch_t* pwr;

    /* check if shutterSpeed is within camera range and smaller than simulation stepTime */
    if(shutterSpeed > 1.0 || shutterSpeed < 0.000020 || shutterSpeed > stepTime)
    {
        printf("ERROR --> Shutter speed value invalid\n");
        return 0;
    }
    
    /* determine current feature mode and set to manual*/
    err = dc1394_feature_set_mode(camera,DC1394_FEATURE_SHUTTER,DC1394_FEATURE_MODE_MANUAL);
    if(err != DC1394_SUCCESS)
    {
        printf("ERROR --> Could not set shutter speed mode to manual\n");
        return 0;
    }
    
    /* determine current power mode for absolute control and set to on */
    err = dc1394_feature_set_absolute_control(camera,DC1394_FEATURE_SHUTTER,DC1394_ON);
    if(err != DC1394_SUCCESS)
    {
        printf("ERROR --> Could not set shutter speed absolute control to on\n");
        return 0;
    }
    
    /* set the requested value */
    err = dc1394_feature_set_absolute_value(camera, DC1394_FEATURE_SHUTTER, shutterSpeed);
    if(err == DC1394_SUCCESS)
        return 1;
    else
    {
        printf("ERROR --> Could not set shutter speed value\n");
        return 0;
    }

    return 1;
    
}

int setShutterSpeedAbs_old(float shutterSpeed, float stepTime)
{
    /* this function was working, but suddenly resulting in segmentation faults when anything is requested from camera */
    dc1394error_t err;
    dc1394feature_mode_t* mode;
    dc1394switch_t* pwr;

    /* check if shutterSpeed is within camera range and smaller than simulation stepTime */
    if(shutterSpeed > 1.0 || shutterSpeed < 0.000020 || shutterSpeed > stepTime)
    {
        printf("ERROR --> Shutter speed value invalid\n");
        return 0;
    }
    
    /* determine current power setting and set to ON */
    err = dc1394_feature_get_power(camera, DC1394_FEATURE_SHUTTER, pwr);
    if(err != DC1394_SUCCESS || *pwr != DC1394_ON)
    {
        err = dc1394_feature_set_power(camera,DC1394_FEATURE_SHUTTER,DC1394_ON);
        if(err != DC1394_SUCCESS)
        {
            printf("ERROR --> Could not set shutter speed power to on\n");
            return 0;
        }
    }
    
    /* determine current feature mode and set to manual*/
    err = dc1394_feature_get_mode(camera,DC1394_FEATURE_SHUTTER,mode);
    if(err != DC1394_SUCCESS || *mode != DC1394_FEATURE_MODE_MANUAL)
    {
        err = dc1394_feature_set_mode(camera,DC1394_FEATURE_SHUTTER,DC1394_FEATURE_MODE_MANUAL);
        if(err != DC1394_SUCCESS)
        {
            printf("ERROR --> Could not set shutter speed mode to manual\n");
            return 0;
        }
    }
    
    /* determine current power mode for absolute control and set to on */
    err = dc1394_feature_get_absolute_control(camera, DC1394_FEATURE_SHUTTER, pwr);
    if(err != DC1394_SUCCESS || *pwr != DC1394_ON)
    {
        err = dc1394_feature_set_absolute_control(camera,DC1394_FEATURE_SHUTTER,DC1394_ON);
        if(err != DC1394_SUCCESS)
        {
            printf("ERROR --> Could not set shutter speed absolute control to on\n");
            return 0;
        }
    }
    
    /* set the requested value */
    err = dc1394_feature_set_absolute_value(camera, DC1394_FEATURE_SHUTTER, shutterSpeed);
    if(err == DC1394_SUCCESS)
        return 1;
    else
    {
        printf("ERROR --> Could not set shutter speed value\n");
        return 0;
    }

    return 1;
    
}

/*-----------------------------------------------------------------------
 *  set the white balance values
 *    return 1 on success, return 0 on failure
 *-----------------------------------------------------------------------*/
int setWhiteBalance(uint32_t u_b_value, uint32_t v_r_value)
{
    dc1394error_t err;
    dc1394feature_mode_t* mode;

    /* check if values are within camera range */
    if(u_b_value > 255 || u_b_value < 0 || v_r_value > 255 || v_r_value < 0)
    {
        printf("ERROR --> White balance value(s) invalid\n");
        return 0;
    }
    
    /* determine current feature mode and set to manual*/
    err = dc1394_feature_set_mode(camera,DC1394_FEATURE_WHITE_BALANCE,DC1394_FEATURE_MODE_MANUAL);
    if(err != DC1394_SUCCESS)
    {
        printf("ERROR --> Could not set white balance mode to manual\n");
        return 0;
    }
    
    /* set the requested value */
    err = dc1394_feature_whitebalance_set_value(camera, u_b_value, v_r_value);
    if(err == DC1394_SUCCESS)
        return 1;
    else
    {
        printf("ERROR --> Could not set white balance values\n");
        return 0;
    }
    
    

    return 1;
    
}

/*-----------------------------------------------------------------------
 *  set the gain value
 *    return 1 on success, return 0 on failure
 *-----------------------------------------------------------------------*/
int setGain(uint32_t gain)
{
    dc1394error_t err;
    dc1394feature_mode_t* mode;
    dc1394switch_t* pwr;

    /* check if values are within camera range */
    if(gain > 48 || gain < 0)
    {
        printf("ERROR --> Gain value invalid\n");
        return 0;
    }
    
    /* turn feature on */
    err = dc1394_feature_set_power(camera, DC1394_FEATURE_GAIN, DC1394_ON);
    if(err != DC1394_SUCCESS)
    {
        printf("ERROR --> Could not set gain power to on\n");
        return 0;
    }
    
    /* set feature mode to manual*/
    err = dc1394_feature_set_mode(camera, DC1394_FEATURE_GAIN, DC1394_FEATURE_MODE_MANUAL);
    if(err != DC1394_SUCCESS)
    {
        printf("ERROR --> Could not set white balance mode to manual\n");
        return 0;
    }
    
    /* set the requested value */
    err = dc1394_feature_set_value(camera, DC1394_FEATURE_GAIN, gain);
    if(err == DC1394_SUCCESS)
        return 1;
    else
    {
        printf("ERROR --> Could not set gain values\n");
        return 0;
    }
    
    return 1;
    
}
