/*
 * Grab an image using libdc1394
 *
 * Written by Damien Douxchamps <ddouxchamps@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Description:
 *
 *  An extension to the original grab image sample program.  This demonstrates
 *  how to collect more detailed information on the various modes of the
 *  camera, convert one image format to another, and waiting so that the
 *  camera has time to capture the image before trying to write it out.
 */

/* From: http://libdc1394-22.sourcearchive.com/documentation/2.0.2/grab__color__image2_8c-source.html
 * Compile: gcc -o grab_color_image2 grab_color_image2.c -ldc1394
 */
 
#include <stdio.h>
#include <stdint.h>
#include <dc1394/dc1394.h>
#include <stdlib.h>
#include <inttypes.h>

#define FRAME_WIDTH 160*4
#define FRAME_HEIGHT 120*4

static dc1394camera_t *camera;
static dc1394_t * d;

int dummy();

/*-----------------------------------------------------------------------
 *  Grabs one (the next in queu) Frame from buffer
 *-----------------------------------------------------------------------*/
int grabOneFrame(uint8_t *image, uint64_t *timestamp);

/*-----------------------------------------------------------------------
 *  Grabs the last inputted Frame from buffer
 *-----------------------------------------------------------------------*/
int grabLastFrame(uint8_t *image, uint64_t *timestamp);

/*-----------------------------------------------------------------------
 *  flush the DMA image buffer
 *-----------------------------------------------------------------------*/
void flushBuffer();
/*-----------------------------------------------------------------------
 *  Releases the cameras and exits
 *-----------------------------------------------------------------------*/
void cleanup_and_exit(dc1394camera_t *camera);

/*-----------------------------------------------------------------------
 *  Initializes the camera and have the camera start sending us data
 *-----------------------------------------------------------------------*/
int fireCamInit(int frameRate);

/*-----------------------------------------------------------------------
 *  stop data transmission and close camera
 *-----------------------------------------------------------------------*/
int fireCamTerm();

/*-----------------------------------------------------------------------
 *  set absolute shutter speed
 *-----------------------------------------------------------------------*/
int setShutterSpeedAbs(float shutterSpeed, float stepTime);

/*-----------------------------------------------------------------------
 *  set white balance
 *-----------------------------------------------------------------------*/
int setWhiteBalance(uint32_t u_b_value, uint32_t v_r_value);

/*-----------------------------------------------------------------------
 *  set the gain value
 *-----------------------------------------------------------------------*/
int setGain(uint32_t gain);
