/*
 * sfuntmpl_basic.c: Basic 'C' template for a level 2 S-function.
 *
 *  -------------------------------------------------------------------------
 *  | See matlabroot/simulink/src/sfuntmpl_doc.c for a more detailed template |
 *  -------------------------------------------------------------------------
 *
 * Copyright 1990-2002 The MathWorks, Inc.
 * $Revision: 1.27.4.2 $
 */


/*
 * You must specify the S_FUNCTION_NAME as the name of your S-function
 * (i.e. replace sfuntmpl_basic with the name of your S-function).
 */

#define S_FUNCTION_NAME  save_signals
#define S_FUNCTION_LEVEL 2

#define FILE_NAME "PITON_signalLog.dat"

/*
 * Need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */
#include "simstruc.h"



/* Error handling
 * --------------
 *
 * You should use the following technique to report errors encountered within
 * an S-function:
 *
 *       ssSetErrorStatus(S,"Error encountered due to ...");
 *       return;
 *
 * Note that the 2nd argument to ssSetErrorStatus must be persistent memory.
 * It cannot be a local variable. For example the following will cause
 * unpredictable errors:
 *
 *      mdlOutputs()
 *      {
 *         char msg[256];         {ILLEGAL: to fix use "static char msg[256];"}
 *         sprintf(msg,"Error due to %s", string);
 *         ssSetErrorStatus(S,msg);
 *         return;
 *      }
 *
 * See matlabroot/simulink/src/sfuntmpl_doc.c for more details.
 */

/*====================*
 * S-function methods *
 *====================*/

/* Function: mdlInitializeSizes ===============================================
 * Abstract:
 *    The sizes information is used by Simulink to determine the S-function
 *    block's characteristics (number of inputs, outputs, states, etc.).
 */
static void mdlInitializeSizes(SimStruct *S)
{
    /* See sfuntmpl_doc.c for more details on the macros below */

    ssSetNumSFcnParams(S, 1);  /* Number of expected parameters */
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        /* Return if number of expected != number of actual parameters */
        return;
    }

    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if(!ssSetNumInputPorts(S, 2)) return;
    ssSetInputPortWidth(S,0,1);
    ssSetInputPortRequiredContiguous(S, 0, true); /*direct input signal access*/
    if(!ssSetInputPortDimensionInfo( S, 1, DYNAMIC_DIMENSION)) return;
    ssSetInputPortRequiredContiguous(S, 1, true); /*direct input signal access*/
    /*
     * Set direct feedthrough flag (1=yes, 0=no).
     * A port has direct feedthrough if the input is used in either
     * the mdlOutputs or mdlGetTimeOfNextVarHit functions.
     * See matlabroot/simulink/src/sfuntmpl_directfeed.txt.
     */
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);

    if (!ssSetNumOutputPorts(S, 0)) return;
    
    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* set work vectors*/
    ssSetNumDWork(S,1);
    ssSetDWorkWidth(S, 0, 2);
    ssSetDWorkDataType(S, 0, SS_DOUBLE);
    
    /* Specify the sim state compliance to be same as a built-in block */
    ssSetSimStateCompliance(S, USE_DEFAULT_SIM_STATE);

    ssSetOptions(S, 0);
}



/* Function: mdlInitializeSampleTimes =========================================
 * Abstract:
 *    This function is used to specify the sample time(s) for your
 *    S-function. You must register the same number of sample times as
 *    specified in ssSetNumSampleTimes.
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);

}



#undef MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
  /* Function: mdlInitializeConditions ========================================
   * Abstract:
   *    In this function, you should initialize the continuous and discrete
   *    states for your S-function block.  The initial states are placed
   *    in the state vector, ssGetContStates(S) or ssGetRealDiscStates(S).
   *    You can also perform any other initialization activities that your
   *    S-function may require. Note, this routine will be called at the
   *    start of simulation and if it is present in an enabled subsystem
   *    configured to reset states, it will be call when the enabled subsystem
   *    restarts execution to reset the states.
   */
  static void mdlInitializeConditions(SimStruct *S)
  {
  }
#endif /* MDL_INITIALIZE_CONDITIONS */



#define MDL_START  /* Change to #undef to remove function */
#if defined(MDL_START) 
  /* Function: mdlStart =======================================================
   * Abstract:
   *    This function is called once at start of model execution. If you
   *    have states that should be initialized once, this is the place
   *    to do it.
   */
  static void mdlStart(SimStruct *S)
  {
    /* file access */
    FILE* dataFile;
    
    /* open file 
     * - opening the data file with "w+" overwrites any existing file
     */
    dataFile = fopen(FILE_NAME, "w+");
    
    if(dataFile == NULL)
    {
        printf("saveSignals: Can't create output file. Is executable running as superuser?\n");
        ssSetErrorStatus(S,"saveSignals: Can't create output file. Is executable running as superuser?");
        return;
    }
    else
    {
        fclose(dataFile);
    }
    
    /* initialize work vector */ 
    real_T* t = ssGetDWork(S, 0);
    t[0] = 0.0;
  }
#endif /*  MDL_START */



/* Function: mdlOutputs =======================================================
 * Abstract:
 *    In this function, you compute the outputs of your S-function
 *    block.
 */
static void mdlOutputs(SimStruct *S, int_T tid)
{
    /* input port access */
    const real_T *tImg = (const real_T*) ssGetInputPortSignal(S,0);
    
    /* start time and image time storage */ 
    real_T* t = ssGetDWork(S, 0);
                
    if(*tImg != t[1])
    {
        /* parameter save_current_linux_time */
        const real_T *pPtr0 = mxGetPr(ssGetSFcnParam(S,0));
        const uint8_T saveTime = (uint8_T)*pPtr0;
    
        /* input port access */
        const real_T *u = (const real_T*) ssGetInputPortSignal(S,1);
        int_T uWidth = ssGetInputPortWidth(S,1);

        /* file access */
        FILE* dataFile;

        int_T i;

        /* open file */
        dataFile = fopen(FILE_NAME, "a");

        if(dataFile == NULL)
        {
            printf("saveSignals: Can't open output file. Is executable running as superuser?\n");
            ssSetErrorStatus(S,"saveSignals: Can't open output file. Is executable running as superuser?");
            return;
        }
        else
        {
            for(i = 0; i < uWidth; i++)
            {
                if(i == uWidth -1 && saveTime)
                {
                    /* linux timestamp variables */
                    struct timeval tv;
                    real_T curTimeSec;

                    /* get current time */
                    gettimeofday(&tv, NULL); 
                    curTimeSec=tv.tv_sec*1000000 + tv.tv_usec;

                    /* save start time */
                    if(t[0] == 0.0)
                        t[0] = tv.tv_sec+(tv.tv_usec/1000000.0);
                    
                    fprintf(dataFile,"%f, %f\n", u[i], (double) tv.tv_sec+(tv.tv_usec/1000000.0) - t[0]);
                }
                else if(i == uWidth -1)
                    fprintf(dataFile,"%f\n", u[i]);
                else
                    fprintf(dataFile,"%f, ",u[i]);
            }
            fclose(dataFile);
        }
        
        /* save new image time */
        t[1] = *tImg;
    }
    
}



#undef MDL_UPDATE  /* Change to #undef to remove function */
#if defined(MDL_UPDATE)
  /* Function: mdlUpdate ======================================================
   * Abstract:
   *    This function is called once for every major integration time step.
   *    Discrete states are typically updated here, but this function is useful
   *    for performing any tasks that should only take place once per
   *    integration step.
   */
  static void mdlUpdate(SimStruct *S, int_T tid)
  {
  }
#endif /* MDL_UPDATE */



#undef MDL_DERIVATIVES  /* Change to #undef to remove function */
#if defined(MDL_DERIVATIVES)
  /* Function: mdlDerivatives =================================================
   * Abstract:
   *    In this function, you compute the S-function block's derivatives.
   *    The derivatives are placed in the derivative vector, ssGetdX(S).
   */
  static void mdlDerivatives(SimStruct *S)
  {
  }
#endif /* MDL_DERIVATIVES */



/* Function: mdlTerminate =====================================================
 * Abstract:
 *    In this function, you should perform any actions that are necessary
 *    at the termination of a simulation.  For example, if memory was
 *    allocated in mdlStart, this is the place to free it.
 */
static void mdlTerminate(SimStruct *S)
{
}


/*======================================================*
 * See sfuntmpl_doc.c for the optional S-function methods *
 *======================================================*/

/*=============================*
 * Required S-function trailer *
 *=============================*/

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
